//+------------------------------------------------------------------+
//|                                              CSampleStrategy.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#include <Object.mqh>

#define POSITION_NEUTRAL   0     //  
#define POSITION_LONG      1     //  
#define POSITION_SHORT    -1     //  

#define SIGNAL_OPEN_LONG    10   //     
#define SIGNAL_OPEN_SHORT  -10   //     
#define SIGNAL_CLOSE_LONG   -1   //     
#define SIGNAL_CLOSE_SHORT   1   //     
//+------------------------------------------------------------------+
//|                  |
//+------------------------------------------------------------------+
struct virtual_position
  {
   string            symbol;            // 
   int               direction;         //    (0-  ,+1 ,-1 )
   double            volume;            //    
   double            profit;            //       
   double            stop_loss;         // Stop Loss  
   double            take_profit;       // Take Profit  
   datetime          time_open;         //      
   datetime          time_close;        //      
   double            price_open;        //    
   double            price_close;       //    
   double            price_highest;     //     
   double            price_lowest;      //     
   double            entry_eff;         //  
   double            exit_eff;          //  
   double            trade_eff;         //  
  };
//+------------------------------------------------------------------+
//|  CSampleStrategy                                            |
//+------------------------------------------------------------------+
class CSampleStrategy: public CObject
  {
protected:
   int               m_strategy_id;            // ID 
   string            m_strategy_symbol;        //  
   string            m_strategy_name;          //  
   string            m_strategy_comment;       // 

   MqlTick           m_price_last;             //  
   MqlRates          m_rates[];                //    
   bool              m_virtual_trade_allowed;  //     
   int               m_current_signal_state;   //   
   double            m_current_trade_volume;   //    
   double            m_initial_balance;        //   (  ,   10000)
   int               m_sl_points;              // Stop Loss
   int               m_tp_points;              // Take Profit

   virtual_position  m_position;               //  
   virtual_position  m_deals_history[];        //  
   int               m_virtual_deals_total;    //   

   double            m_virtual_balance;           // "" 
   double            m_virtual_equity;            // "" 
   double            m_virtual_cumulative_profit; //  "" 
   double            m_virtual_profit;            //    "" 

   //   ,     
   bool              CheckVirtual_Stops(virtual_position &position);
   //    
   void              RecalcPositionProperties(virtual_position &position);
   //          
   void              Position_RefreshInfo(virtual_position &position);
   //     
   void              Position_OpenShort(virtual_position &position);
   //       
   void              Position_CloseShort(virtual_position &position);
   //     
   void              Position_OpenLong(virtual_position &position);
   //     
   void              Position_CloseLong(virtual_position &position);
   //      
   void              Position_CloseOpenedPosition(virtual_position &position);
   //      m_deals_history[] ( )
   void              AddDealToHistory(virtual_position &position);
   //     ,     
   virtual double    MoneyManagement_CalculateLots(double trade_volume);
public:
   // 
   void              CSampleStrategy();
   // 
   void             ~CSampleStrategy();

   //    
   double            GetVirtualBalance() { return(m_virtual_balance); }
   //    
   double            GetVirtualEquity() { return(m_virtual_equity); }
   //      
   double            GetVirtualProfit() { return(m_virtual_profit); }

   // Stop Loss  Take Profit  
   void              Set_Stops(int tp,int sl) {m_tp_points=tp; m_sl_points=sl;};
   //   
   void              SetLots(double trade_volume) {m_current_trade_volume=trade_volume;};
   //   
   double            GetCurrentLots() { return(m_current_trade_volume); }

   //   
   string            StrategyName() { return(m_strategy_name); }
   //  ID 
   int               StrategyID() { return(m_strategy_id); }
   //   
   string            StrategyComment() { return(m_strategy_comment); }
   //     (, , ID, )
   void              SetStrategyInfo(string symbol,string name,int id,string comment);

   //     (  )
   void              SetVirtualTradeFlag(bool pFlag) { m_virtual_trade_allowed=pFlag; };
   //     
   bool              IsVirtualTradeAllowed() { return(m_virtual_trade_allowed); };

   //    
   int               GetSignalState();
   //     ( ,    )
   void              SetSignalState(int state);
   //          
   void              ProceedSignalState(virtual_position &position);

   //    "" 
   void              SetVirtualCumulativeProfit(double cumulative_profit) { m_virtual_cumulative_profit=cumulative_profit; };

   //   ()
   double            StrategyPerformance();

   //  
   void              UpdatePositionData();
   //   
   void              CloseVirtualPosition();
   //    
   int               PositionDirection();
   //  
   virtual int       Initialization() {return(0);};
   //    
   virtual bool      CheckTradeConditions() {return(false);};
   //  
   virtual int       Deinitialization() {return(0);};

   //    
   void              SaveVirtualDeals(string file_name);
  };
//+------------------------------------------------------------------+
//|   CSampleStrategy                               |
//+------------------------------------------------------------------+
void CSampleStrategy::CSampleStrategy()
  {
   ZeroMemory(m_position);              //  m_position
   m_virtual_cumulative_profit=0;       //   
   m_virtual_profit=0;                  //  
   m_current_trade_volume=0.1;          //    
   m_strategy_id=0;                     //   ID=0
   m_strategy_name="Sample Strategy";   //   "Sample Strategy"
   m_current_signal_state=0;            //   
   m_virtual_trade_allowed=true;        //     
   m_initial_balance=10000;             //   
   m_virtual_balance=m_initial_balance; //   
   m_virtual_equity=m_initial_balance;  //    (equity)
   m_sl_points=0;                       //  Stop Loss ( )  
   m_tp_points=0;                       //  Take Profit ( )  
  }
//+------------------------------------------------------------------+
//|   CSampleStrategy                               |
//+------------------------------------------------------------------+ 
void CSampleStrategy::~CSampleStrategy()
  {
//  
   ArrayResize(m_rates,0);
   ArrayResize(m_deals_history,0);
  }
//+------------------------------------------------------------------+
//|                                    |
//+------------------------------------------------------------------+
void CSampleStrategy::UpdatePositionData()
  {
   RecalcPositionProperties(m_position);
  }
//+------------------------------------------------------------------+
//|                                 |
//+------------------------------------------------------------------+
void CSampleStrategy::CloseVirtualPosition()
  {
   Position_CloseOpenedPosition(m_position);
  }
//+------------------------------------------------------------------+
//|                               |
//+------------------------------------------------------------------+
int CSampleStrategy::PositionDirection()
  {
   return(m_position.direction);
  }
//+------------------------------------------------------------------+
//|              |
//|                   |
//|       :                   |
//|  m_virtual_balance  m_virtual_equity                           |
//|    (  m_deals_history)           |
//|  ,                                            |
//|                     |
//|     :  return(trade_volume);         |
//+------------------------------------------------------------------+ 
double CSampleStrategy::MoneyManagement_CalculateLots(double trade_volume)
  {
//   
   return(trade_volume);
  }
//+------------------------------------------------------------------+
//|  StrategyPerformance -    |
//+------------------------------------------------------------------+ 
double CSampleStrategy::StrategyPerformance()
  {
//  
//       
//      , 
//..   
   double performance=(m_virtual_equity-m_initial_balance);
   return(performance);
/*
  //  ,         
   if(m_virtual_deals_total>0)
     {
      int avdeals=MathRound(MathMin(3,m_virtual_deals_total));
      double sumprofit=0;
      for(int j=0; j<avdeals; j++)
        {
         sumprofit+=m_deals_history[m_virtual_deals_total-1-j].profit;
        }
      sumprofit=sumprofit/avdeals;
      performance=performance*sumprofit;
     }
     return(performance);
*/
  }
//+------------------------------------------------------------------+
//|     (, ID, )      |
//+------------------------------------------------------------------+
void  CSampleStrategy::SetStrategyInfo(string symbol,string name,int id,string comment)
  {
   //
   m_strategy_symbol=symbol;
   //ID
   m_strategy_id=id;
   //
   m_strategy_name=name;
   //
   m_strategy_comment=comment;
  // 
   m_position.symbol=m_strategy_symbol;
  }
//+------------------------------------------------------------------+
//| ""    ,                |
//|                                     |
//|      m_deals_history                      |
//+------------------------------------------------------------------+
void  CSampleStrategy::AddDealToHistory(virtual_position &position)
  {
   //     
   m_virtual_deals_total=ArraySize(m_deals_history);
   //   1
   m_virtual_deals_total++;
   //   m_deals_history  1
   ArrayResize(m_deals_history,m_virtual_deals_total);
   //  position    
   m_deals_history[m_virtual_deals_total-1].symbol=position.symbol;
   m_deals_history[m_virtual_deals_total-1].direction=position.direction;
   m_deals_history[m_virtual_deals_total-1].volume=position.volume;
   m_deals_history[m_virtual_deals_total-1].stop_loss=position.stop_loss;
   m_deals_history[m_virtual_deals_total-1].take_profit=position.take_profit;
   m_deals_history[m_virtual_deals_total-1].profit = position.profit;
   m_deals_history[m_virtual_deals_total-1].volume = position.volume;
   m_deals_history[m_virtual_deals_total-1].time_open=position.time_open;
   m_deals_history[m_virtual_deals_total-1].time_close = position.time_close;
   m_deals_history[m_virtual_deals_total-1].price_open = position.price_open;
   m_deals_history[m_virtual_deals_total-1].price_close= position.price_close;
   m_deals_history[m_virtual_deals_total-1].price_highest= position.price_highest;
   m_deals_history[m_virtual_deals_total-1].price_lowest = position.price_lowest;
   m_deals_history[m_virtual_deals_total-1].entry_eff= position.entry_eff;
   m_deals_history[m_virtual_deals_total-1].exit_eff = position.exit_eff;
   m_deals_history[m_virtual_deals_total-1].trade_eff= position.trade_eff;
   //  
   m_virtual_profit=position.profit;
   //       
   m_virtual_cumulative_profit=m_virtual_cumulative_profit+m_virtual_profit;
   //   ( )
   position.direction=0;
   //  
   position.profit=0;
  }
//+------------------------------------------------------------------+
//|    ,       |
//+------------------------------------------------------------------+
bool CSampleStrategy::CheckVirtual_Stops(virtual_position &position)
  {
   if(position.direction==0) {return(false);}
   //  ,  ,      
   if(position.direction==POSITION_LONG)
     {
      if((position.stop_loss>0.0) && (position.price_close<=position.stop_loss))
        {
         //Print("Close LONG: Stop Loss: ",position.stop_loss," Price:",position.price_close);
         Position_CloseLong(position);
         return(true);
        }
      if((position.take_profit>0.0) && (position.price_close>=position.take_profit))
        {
         //Print("Close LONG: Take Profit: ",position.take_profit," Price:",position.price_close);
         Position_CloseLong(position);
         return(true);
        };
     }
   if(position.direction==POSITION_SHORT)
     {
      if((position.stop_loss>0.0) && (position.price_close>=position.stop_loss))
        {
         //Print("Close SHORT: Stop Loss: ",position.stop_loss," Price:",position.price_close);
         Position_CloseShort(position);
         return(true);
        }
      if((position.take_profit>0.0) && (position.price_close<=position.take_profit))
        {
         //Print("Close SHORT: ",position.take_profit," Price:",position.price_close);
         Position_CloseShort(position);
         return(true);
        }
     }
   return(false);
  }
//+------------------------------------------------------------------+
//|                  |
//+------------------------------------------------------------------+
void CSampleStrategy::Position_RefreshInfo(virtual_position &position)
  {
   //     
   SymbolInfoTick(position.symbol,m_price_last);

   //  ,     
   if(position.direction!=0)
     {
      position.time_close=0;
      position.price_highest=MathMax(position.price_highest,m_price_last.ask);
      position.price_lowest=MathMin(position.price_lowest,m_price_last.bid);

      double xfactor;
      if((position.price_highest-position.price_lowest)==0)
        {
         //Print("BID=ASK?","Bid=",m_price_last.bid," Ask=",m_price_last.ask);
         return;
        }
      xfactor=1.0/(position.price_highest-position.price_lowest);

      //   
      if(position.direction==POSITION_LONG)
        {
         position.price_close=m_price_last.bid;
         position.entry_eff=(position.price_highest-position.price_open)*xfactor;
         position.exit_eff=(position.price_close-position.price_lowest)*xfactor;
         position.trade_eff=(position.price_close-position.price_open)*xfactor;
         position.profit=position.volume*(position.price_close-position.price_open)/_Point;
        }
      //   
      if(position.direction==POSITION_SHORT)
        {
         position.price_close=m_price_last.ask;
         position.entry_eff=(position.price_open-position.price_lowest)*xfactor;
         position.exit_eff=(position.price_highest-position.price_close)*xfactor;
         position.trade_eff=(position.price_open-position.price_close)*xfactor;
         position.profit=position.volume*(position.price_open-position.price_close)/_Point;
        }
      CheckVirtual_Stops(position);
     }
  }
//+------------------------------------------------------------------+
//|                    |
//+------------------------------------------------------------------+
void CSampleStrategy::RecalcPositionProperties(virtual_position &position)
  {
   //   
   Position_RefreshInfo(m_position);
   //   ,  
   if(m_position.direction!=0)
     {
      m_virtual_profit=m_position.profit;
     }
   //string s1="RecalcPositionProperties:"+StrategyName()+" "+m_virtual_equity+" Balance="+m_virtual_balance+" m_virtual_cumulative_profit="+m_virtual_cumulative_profit+" "+m_virtual_profit;

   //  equity  
   m_virtual_equity=m_initial_balance+m_virtual_cumulative_profit+m_virtual_profit;
   m_virtual_balance=m_initial_balance+m_virtual_cumulative_profit;

   //   s1=s1+" new value="+m_virtual_equity+" new balance="+m_virtual_balance;  
   //   Print(s1);
  }
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
void CSampleStrategy::Position_CloseShort(virtual_position &position)
  {
   //,    
   if(!m_virtual_trade_allowed) {return;}
   //,       POSITION_SHORT
   if(position.direction!=POSITION_SHORT){return;}
   //    
   position.time_close=TimeCurrent();
   //       
   AddDealToHistory(position);
  }
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
void CSampleStrategy::Position_OpenLong(virtual_position &position)
  {
   //,    
   if(!m_virtual_trade_allowed) {return;}
   //   
   position.direction=POSITION_LONG;

   //      Money Management
   m_current_trade_volume=MoneyManagement_CalculateLots(m_current_trade_volume);
   //   
   position.volume=m_current_trade_volume;

   //    
   position.time_open=TimeCurrent();
   //       
   position.price_open=m_price_last.ask;

   // Stop Loss    
   if(m_sl_points>0) position.stop_loss=position.price_open+m_sl_points*_Point; else position.stop_loss=0;
   // Take Profit    
   if(m_tp_points>0) position.take_profit=position.price_open-m_tp_points*_Point; else position.take_profit=0;

   //Print("Position_OpenLong: SL=",position.stop_loss," TP=",position.take_profit);

   //      (  )  
   position.price_close=m_price_last.bid;
   //    
   position.time_close=0;
   // 
   position.profit=position.volume*(position.price_close-position.price_open)/_Point;
   //  
   position.price_highest=m_price_last.ask;
   //  
   position.price_lowest=m_price_last.bid;
   //      
   RecalcPositionProperties(position);
  }
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
void CSampleStrategy::Position_CloseLong(virtual_position &position)
  {
   //,    
   if(!m_virtual_trade_allowed) {return;}
   //,       POSITION_LONG
   if(position.direction!=POSITION_LONG){return;}
   //    
   position.time_close=TimeCurrent();
   //       
   AddDealToHistory(position);
  }
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
void CSampleStrategy::Position_OpenShort(virtual_position &position)
  {
   //,    
   if(!m_virtual_trade_allowed) {return;}
   //   
   position.direction=POSITION_SHORT;

   //      Money Management
   m_current_trade_volume=MoneyManagement_CalculateLots(m_current_trade_volume);
   //   
   position.volume=m_current_trade_volume;

   //    
   position.time_open=TimeCurrent();
   //    
   position.price_open=m_price_last.bid;
   //      (  )  
   position.price_close=m_price_last.ask;

   // Stop Loss    
   if(m_sl_points>0) position.stop_loss=position.price_open+m_sl_points*_Point; else position.stop_loss=0;
   // Take Profit    
   if(m_tp_points>0) position.take_profit=position.price_open-m_tp_points*_Point; else position.take_profit=0;

   //Print("Position_OpenShort: SL=",position.stop_loss," TP=",position.take_profit);

   //    
   position.time_close=0;
   // 
   position.profit=position.volume*(position.price_open-position.price_close)/_Point;
   //  
   position.price_highest=m_price_last.ask;
   //  
   position.price_lowest=m_price_last.bid;
   //      
   RecalcPositionProperties(position);
  }
//+------------------------------------------------------------------+
//|                               |
//+------------------------------------------------------------------+
void CSampleStrategy::Position_CloseOpenedPosition(virtual_position &position)
  {
   if(position.direction!=POSITION_NEUTRAL)
     {
      if(position.direction==POSITION_SHORT) Position_CloseShort(position);
      if(position.direction==POSITION_LONG)  Position_CloseLong(position);
     }
  }
//+------------------------------------------------------------------+
//|    position               |
//|   m_current_signal_state                              |
//+------------------------------------------------------------------+
void  CSampleStrategy::ProceedSignalState(virtual_position &position)
  {
   if(position.direction!=POSITION_NEUTRAL) //    
     {
      switch(m_current_signal_state)
        {
         case SIGNAL_CLOSE_SHORT:   { Position_CloseShort(position); break; }
         case SIGNAL_CLOSE_LONG:    { Position_CloseLong(position); break;}
        }
     }
   else                             //    
     {
      switch(m_current_signal_state)
        {
         case SIGNAL_OPEN_LONG:   { Position_OpenLong(position); break; }
         case SIGNAL_OPEN_SHORT:  { Position_OpenShort(position); break;}
        }
     }
  }
//+------------------------------------------------------------------+
//|                               |
//+------------------------------------------------------------------+
int CSampleStrategy::GetSignalState()
  { return(m_current_signal_state); }
//+------------------------------------------------------------------+
//|                            |
//|  ,             |
//+------------------------------------------------------------------+
void  CSampleStrategy::SetSignalState(int state)
  {
   if(m_current_signal_state!=state)
     {
      m_current_signal_state=state;
      ProceedSignalState(m_position);
     }
  };
//+------------------------------------------------------------------+
//|                     |
//+------------------------------------------------------------------+
void CSampleStrategy::SaveVirtualDeals(string file_name)
  {
   //   , 
   if(m_virtual_deals_total==0) {return;};
   //   
   int filehandle=FileOpen(file_name,FILE_WRITE|FILE_CSV);
   //  
   FileWrite(filehandle,
             "Symbol",
             "Direction",
             "Profit",
             "Volume",
             "OpenTime",
             "CloseTime",
             "OpenPrice",
             "ClosePrice",
             "HighestPrice",
             "LowestPrice",
             "Entry_Eff",
             "Exit_Eff",
             "Trade_Eff");
   //  
   for(int i=0; i<m_virtual_deals_total; i++)
     {
      FileWrite(filehandle,
                m_deals_history[i].symbol,
                m_deals_history[i].direction,
                m_deals_history[i].profit,
                m_deals_history[i].volume,
                m_deals_history[i].time_open,
                m_deals_history[i].time_close,
                m_deals_history[i].price_open,
                m_deals_history[i].price_close,
                m_deals_history[i].price_highest,
                m_deals_history[i].price_lowest,
                m_deals_history[i].entry_eff,
                m_deals_history[i].exit_eff,
                m_deals_history[i].trade_eff);
     }
   FileClose(filehandle);
  }
//+------------------------------------------------------------------+
//|                                |
//+------------------------------------------------------------------+
bool IsNewBar()
  {
   static datetime old_time;
   datetime new_time[1];
   int copied=CopyTime(_Symbol,_Period,0,1,new_time);
   if(copied>0)
     {
      if(old_time!=new_time[0])
        {
         old_time=new_time[0];
         return(true);
        }
     }
   else
     {
      Alert("  ,   =",GetLastError());
      ResetLastError();
     }
   return(false);
  }
//+------------------------------------------------------------------+