//+------------------------------------------------------------------+
//|                                                  CStrategyMA.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#include <CSampleStrategy.mqh>
//+------------------------------------------------------------------+
//|  CStrategyMA                |
//|                                      |
//+------------------------------------------------------------------+
class CStrategyMA : public CSampleStrategy
  {
protected:
   int               m_handle;     //   Moving Average (iMA)
   int               m_period;     //   Moving Average
   double            m_values[];   //     
public:
   //  
   int               Initialization(int period,bool virtual_trade_flag);
   //  
   int               Deinitialization();
   //       
   bool              CheckTradeConditions();
  };
//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
int CStrategyMA::Initialization(int period,bool virtual_trade_flag)
  {
   //   
   m_period=period;
   //     
   SetVirtualTradeFlag(virtual_trade_flag);

   //     
   ArraySetAsSeries(m_rates,true);
   ArraySetAsSeries(m_values,true);
   
   //   
   m_handle=iMA(_Symbol,_Period,m_period,0,MODE_EMA,PRICE_CLOSE);
   if(m_handle<0)
     {
      Alert("    MA -  : ",GetLastError(),"!!");
      return(-1);
     }

   return(0);
  }
//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
int CStrategyMA::Deinitialization()
  {
   Position_CloseOpenedPosition(m_position);
   IndicatorRelease(m_handle);
   return(0);
  };
//+------------------------------------------------------------------+
//|                |
//+------------------------------------------------------------------+
bool CStrategyMA::CheckTradeConditions()
  {
   RecalcPositionProperties(m_position);
   double p_close;

//---     3- 
   if(CopyRates(_Symbol,_Period,0,3,m_rates)<0)
     {
      Alert("    - :",GetLastError(),"!!");
      return(false);
     }
//       (  1)
   p_close=m_rates[1].close;  //              

   if(CopyBuffer(m_handle,0,0,3,m_values)<0)
     {
      Alert("    Moving Average -  :",GetLastError());
      return(false);
     }

//   1: MA 
   bool buy_condition_1=(m_values[0]>m_values[1]) && (m_values[1]>m_values[2]);
//   2:       MA
   bool buy_condition_2=(p_close>m_values[1]);

//   1: // MA 
   bool sell_condition_1=(m_values[0]<m_values[1]) && (m_values[1]<m_values[2]);
//   2: //     MA   
   bool sell_condition_2=(p_close<m_values[1]);

   int new_state=0;

   if(buy_condition_1  &&  buy_condition_2) new_state=SIGNAL_OPEN_LONG;
   if(sell_condition_1 && sell_condition_2) new_state=SIGNAL_OPEN_SHORT;

   if((GetSignalState()==SIGNAL_OPEN_SHORT) && (buy_condition_1 || buy_condition_2)) new_state=SIGNAL_CLOSE_SHORT;
   if((GetSignalState()==SIGNAL_OPEN_LONG) && (sell_condition_1 || sell_condition_2)) new_state=SIGNAL_CLOSE_LONG;

   if(GetSignalState()!=new_state)
     {
      SetSignalState(new_state);
     }

   return(true);
  };
//+------------------------------------------------------------------+
